/*
* file CcStream.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcStream
*
* (C) Copyright IBM Corporation 2004, 2011.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.Feedback;
import javax.wvcm.Resource;
import javax.wvcm.ResourceList;
import javax.wvcm.Stream;
import javax.wvcm.WvcmException;
import javax.wvcm.Baseline.CompareReport;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.ResourceList.ResponseIterator;

import com.ibm.rational.wvcm.stp.StpActivity;
import com.ibm.rational.wvcm.stp.cc.CcBaseline.CompareFlagEx;

/**
 * <p>
 * A proxy for a ClearCase UCM stream.
 * </p>
 * <p>
 * Each UCM stream represents a separate parallel development effort in
 * ClearCase. Code changes made in one UCM stream are not visbile to other
 * streams until the user chooses to share them via the UCM <i>deliver</i> or
 * <i>rebase</i> operations.
 * </p>
 * <p>
 * A UCM stream has one or more ClearCase views associated with it in which
 * users view and modify their version-controlled resources. The stream
 * determines these views' <i>configuration</i> - the set of components visible
 * in those views and the particular version of each file in each component.
 * This configuration consists of the versions selected by the stream's
 * <i>foundation baseline list</i>, plus the versions in the change sets of the
 * stream's activities.
 * </p>
 * <p>
 * The stream user creates a UCM activity in the stream for each logical task
 * they are working on. When the user checks in a version-controlled resource,
 * the new version is collected in that activity's change set. The user may
 * create as many activities as they need to complete their tasks.
 * </p>
 * <p>
 * Each UCM project has a single <i>integration stream</i> to which the
 * project's <i>development streams</i> deliver their changes. UCM supports
 * multi-level stream hierarchies - development streams that have their own
 * substreams and serve as <i>delivery targets</i> for those substreams.
 * </p>
 */
public interface CcStream extends Stream, CcVobResource {

    /**
     * <p>
     * Create a new UCM stream at the location specified by this proxy. The
     * location should be an object name selector specifying the stream's name
     * and the repository (project VOB) in which to create it.
     * </p>
     * <p>
     * To create an integration stream, set the {@link #PARENT_PROJECT} property
     * to the project in which to create the stream, and set the
     * {@link #IS_INTEGRATION_STREAM} property to "true". A given project can
     * have at most one integration stream.
     * </p>
     * <p>
     * To create a non-integration stream, set the {@link #PARENT_STREAM}
     * property to the integration stream under which to create the stream, and
     * set the {@link #IS_INTEGRATION_STREAM} property to "false".
     * </p>
     * <p>
     * You may also set the following additional stream properties prior to
     * calling this method:
     * <bl>
     * <li> {@link #FOUNDATION_BASELINE_LIST} </li>
     * <li> {@link #DEFAULT_DELIVER_TARGET} </li>
     * <li> {@link #IS_READ_ONLY} </li>
     * <li> {@link #COMMENT} </li>
     * </bl>
     * </p>
     */
    public CcStream doCreateCcStream(Feedback feedback) throws WvcmException;

    /**
     * <p>Create a new UCM stream, allowing the provider to choose the
     * new stream's name.
     * The provider may use the client-specified location if it is valid,
     * but can select a different location if the location is not valid
     * or already identifies an stream.
     * </p>
     * @see #doCreateCcStream(Feedback)
     */
    public CcStream doCreateGeneratedCcStream(Feedback feedback) throws WvcmException;

    /**
     * <p>
     * Compare this CcStream with the specified stream.
     * </p>
     * <p>
     * Specifically, compute the differences between these two streams in
     * terms of baselines, activities, and/or versions. 
     * </p>
     * @param stream stream to compare against
     * @param flags specifies the types of differences to include in the
     *              compare report.
     * @param context optional resource (often CcView) providing context for the
     *                generation of certain properties in the returned report. 
     *                May be <b>null</b>.
     * @param feedback the properties available in the returned proxies.
     * @return a ResponseIterator of CompareReport objects, that enumerate the 
     *         differences between the versions selected by this CcStream and 
     *         the stream argument.
     * @throws WvcmException
     * @see CcBaseline#doCompareReportEx(CcStream, CompareFlagEx[], javax.wvcm.Resource, javax.wvcm.Feedback)
     *      for more information
     */
    public ResponseIterator<CompareReport>
    doCompareReportEx(CcStream stream, CompareFlagEx[] flags, Resource context, Feedback feedback)
        throws WvcmException;

    /**
     * <p>
     * List of all activities in the UCM stream. 
     * </p>
     */
    PropertyName<ResourceList<CcActivity>> ACTIVITY_LIST =
        new PropertyName<ResourceList<CcActivity>>(PROPERTY_NAMESPACE,
                                                   "activity-list");

    /**
     * Get the value of this stream's {@link #ACTIVITY_LIST} property.
     * 
     * @return a CcResourceList containing CcActivity instances, as described above
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    ResourceList<CcActivity> getActivityList() throws WvcmException;

    /**
     * <p>
     * The current user's activities in this UCM stream. The
     * exact semantics of this property differ depending on whether the stream
     * is associated with a ClearQuest-enabled UCM project.
     * </p>
     * <p>
     * If this stream is associated with a ClearQuest-enabled UCM project, the
     * resulting activity list is the result set from the <i>UCMCustomQuery1</i>
     * query executed in the ClearQuest user database to which the UCM project
     * is bound. In the out-of-the-box ClearQuest UCM schema, this query will
     * return all CQ activities that are:
     * </p>
     * <ul>
     * <li>assigned to the current user</li>
     * <li>AND in the ready or active state,</li>
     * <li>AND are NOT already bound to another project</li>
     * </ul>
     * <p>
     * The UCMCustomQuery1 query can be modified by the customer, and if so will
     * return other results.
     * </p>
     * <p>
     * On the other hand, if this stream's project is <i>not</i> ClearQuest-enabled,
     * the resulting activity list is simply all UCM activities in
     * the stream that belong to the current user.
     * </p>
     * <p>
     * In either case, the actual property value is a ResourceList of
     * StpActivity instances.
     * </p>
     * @see com.ibm.rational.wvcm.stp.StpActivity
     */
    PropertyName<ResourceList<StpActivity>> MY_ACTIVITY_LIST =
        new PropertyName<ResourceList<StpActivity>>(PROPERTY_NAMESPACE,
                                                   "my-activity-list");

    /**
     * Get the value of this stream's {@link #MY_ACTIVITY_LIST} property.
     * 
     * @return a ResourceList of StpActivity instances, as described above
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    ResourceList<StpActivity> getMyActivityList() throws WvcmException;
 
    /**
     * <p>
     * This stream's baselines.  A context of a CcComponent may be specified
     * when reading this property in which case the list is limited to baselines
     * for the specified CcComponent.
     * </p>
     * <p>
     * When a stream is baselined, a new baseline is created for each component
     * in the stream's configuration that was changed since the last time the
     * stream was baselined.  If a given component has not changed, no
     * baseline is created for that component.
     * </p>
     */
    PropertyName<ResourceList<CcBaseline>> BASELINE_LIST =
        new PropertyName<ResourceList<CcBaseline>>(PROPERTY_NAMESPACE, "baseline-list");

    /**
     * Get the value of the this proxy's {@link #BASELINE_LIST} property.
     * @return a list of client proxies for this project's baselines
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<CcBaseline> getBaselineList() throws WvcmException;

    /**
     * <p>
     * This stream's latest baselines.  Specifically, the most recent baseline
     * of each component in this stream's configuration that has been modified
     * in this stream or the foundation baseline for components that have
     * not been modified.
     * </p>
     * <p>
     * When a stream is baselined, a new baseline is created for each component
     * in the stream's configuration that was changed since the last time the
     * stream was baselined.  If a given component has not changed, no
     * baseline is created for that component.
     * </p>
     */
    PropertyName<ResourceList<CcBaseline>> LATEST_BASELINE_LIST =
        new PropertyName<ResourceList<CcBaseline>>(PROPERTY_NAMESPACE, "latest-baseline-list");

    /**
     * Get the value of the this proxy's {@link #LATEST_BASELINE_LIST} property.
     * @return a list of client proxies for this project's latest baselines
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<CcBaseline> getLatestBaselineList() throws WvcmException;

    /**
     * The project to which this UCM stream belongs.
     */
    PropertyName<CcProject> PARENT_PROJECT =
        new PropertyName<CcProject>(PROPERTY_NAMESPACE,
                                    "parent-project");

    /**
     * Get the value of this stream's {@link #PARENT_PROJECT} property.
     * 
     * @return stream's parent project
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    CcProject getParentProject() throws WvcmException;

    /**
     * Set the value of this stream's {@link #PARENT_PROJECT} property.
     * This property can only be set at stream creation time.
     * 
     * @param parent
     *            stream's parent project
     */
    void setParentProject(CcProject parent);

    /**
     * This UCM stream's parent stream.  The value of this property is null
     * if this is an integration stream.
     */
    PropertyName<CcStream> PARENT_STREAM = new PropertyName<CcStream>(PROPERTY_NAMESPACE,
            "parent-stream");

    /**
     * Get the value of this stream's {@link #PARENT_STREAM} property.
     * 
     * @return stream's parent stream, or null if this is an integration stream
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    CcStream getParentStream() throws WvcmException;

    /**
     * Set the value of this stream's {@link #PARENT_STREAM} property.
     * This property can only be set at stream creation time.
     * 
     * @param parent
     *            stream's parent stream
     */
    void setParentStream(CcStream parent);

    /** This UCM stream's recommended baselines. */
    PropertyName<ResourceList<CcBaseline>> RECOMMENDED_BASELINE_LIST =
        new PropertyName<ResourceList<CcBaseline>>(PROPERTY_NAMESPACE,
                                                   "recommended-baseline-list");

    /**
     * Get the value of the this proxy's {@link #RECOMMENDED_BASELINE_LIST} property.
     * @return a list of client proxies for this project's recommended baselines
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<CcBaseline> getRecommendedBaselineList() throws WvcmException;

    /** This UCM stream's full closure of recommended baselines. */
    PropertyName<ResourceList<CcBaseline>> RECOMMENDED_BASELINE_LIST_CLOSURE =
        new PropertyName<ResourceList<CcBaseline>>(PROPERTY_NAMESPACE,
                                                   "recommended-baseline-list-closure");

    /**
     * Get the value of the this proxy's {@link #EXPLICIT_BASELINE_LIST} property.
     * @return a list of client proxies for this stream's explicit baselines
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<CcBaseline> getExplicitBaselineList() throws WvcmException;    
 
    
    /** This UCM stream's list of explicit baselines. */
    PropertyName<ResourceList<CcBaseline>> EXPLICIT_BASELINE_LIST =
        new PropertyName<ResourceList<CcBaseline>>(PROPERTY_NAMESPACE,
                                                   "explicit-baseline-list");

    /**
     * Get the value of the this proxy's {@link #RECOMMENDED_BASELINE_LIST_CLOSURE} property.
     * @return a list of client proxies for this project's recommended baselines
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    ResourceList<CcBaseline> getRecommendedBaselineListClosure() throws WvcmException;
    
    /**
     * Set the value of the this proxy's {@link #RECOMMENDED_BASELINE_LIST} property.
     * @param recBls a list of client proxies for this project's recommended baselines
     */
    void setRecommendedBaselineList(ResourceList<CcBaseline> recBls);    
    
    /**
     * The list of substreams of this UCM stream, i.e., the streams
     * for which this stream is the parent.
     */
    PropertyName<ResourceList<CcStream>> SUBSTREAM_LIST =
        new PropertyName<ResourceList<CcStream>>(PROPERTY_NAMESPACE,
                                                 "substream-list");

    /**
     * Get the value of this stream's {@link #SUBSTREAM_LIST} property.
     * 
     * @return a list of client proxies for this stream's substreams
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    ResourceList<CcStream> getSubstreamList() throws WvcmException;

    /** The views associated with this UCM stream. */
    PropertyName<ResourceList<CcView>> VIEW_LIST =
        new PropertyName<ResourceList<CcView>>(PROPERTY_NAMESPACE,
                                               "view-list");

    /**
     * Get the value of this stream's {@link #VIEW_LIST} property.
     * 
     * @return the list of views associated with this stream
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    ResourceList<CcView> getViewList() throws WvcmException;

    /** Is this UCM stream an integration stream? */
    PropertyName<Boolean> IS_INTEGRATION_STREAM =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-integration-stream");

    /**
     * Get the value of this stream's {@link #IS_INTEGRATION_STREAM} property.
     * 
     * @return true if this stream is an integration stream, else false
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    boolean getIsIntegrationStream() throws WvcmException;

    /**
     * Set the value of this stream's {@link #IS_INTEGRATION_STREAM} property.
     * This property can only be set at stream creation time.
     * 
     * @param isInt
     *            true if this stream is an integration stream, else false
     */
    void setIsIntegrationStream(boolean isInt);

    /** Is this a read-only UCM stream? */
    PropertyName<Boolean> IS_READ_ONLY =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-read-only");

    /**
     * Get the value of this stream's {@link #IS_READ_ONLY} property.
     * 
     * @return true if this stream is read-only, else false
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    boolean getIsReadOnly() throws WvcmException;

    /**
     * Set the value of this stream's {@link #IS_READ_ONLY} property.
     * This property can only be set at stream creation time.
     * 
     * @param readOnly
     *            true if this stream is to be read-only, else false
     */
    void setIsReadOnly(boolean readOnly);

    /** Is Delivery allowed with checkouts in this stream */
    PropertyName<Boolean> DELIVER_POLICY_NO_CHECKOUTS_IN_STREAM =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
            "deliver-policy-no-checkouts-stream");     
    
    /**
     * Get the value of this stream's {@link #DELIVER_POLICY_NO_CHECKOUTS_IN_STREAM } property.
     * 
     * @return true if the policy to disallow Deliver with checkouts in this stream is enabled
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    boolean getDeliverPolicyNoCheckoutsInStream() throws WvcmException;    
        
    /**
     * Set the value of this stream's {@link #DELIVER_POLICY_NO_CHECKOUTS_IN_STREAM } property.
     *
     * 
     * @param enablePolicy
     *            true to enable the policy to disallow Deliver with checkouts in this stream
     */    
    void setDeliverPolicyNoCheckoutsInStream(boolean enablePolicy); 
    

    /** Is Delivery allowed with checkouts in selected activities */
    PropertyName<Boolean> DELIVER_POLICY_NO_CHECKOUTS_IN_ACTIVITIES =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
            "deliver-policy-no-checkouts-activities");     
    
    /**
     * Get the value of this stream's {@link #DELIVER_POLICY_NO_CHECKOUTS_IN_ACTIVITIES } property.
     * 
     * @return true if the policy to disallow Deliver with checkouts in selected activities is enabled for this stream
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    boolean getDeliverPolicyNoCheckoutsInActivities() throws WvcmException;    
        
    /**
     * Set the value of this stream's {@link #DELIVER_POLICY_NO_CHECKOUTS_IN_ACTIVITIES } property.
     *
     * 
     * @param enablePolicy
     *            true to enable the policy to disallow Deliver with checkouts in selected activities for this stream
     */    
    void setDeliverPolicyNoCheckoutsInActivities(boolean enablePolicy);    
    
    
    /** This UCM stream's foundation baselines. */
    PropertyName<ResourceList<CcBaseline>> FOUNDATION_BASELINE_LIST =
        new PropertyName<ResourceList<CcBaseline>>(PROPERTY_NAMESPACE,
                                                   "foundation-baseline-list");

    /**
     * Get the value of this stream's {@link #FOUNDATION_BASELINE_LIST}
     * property.
     * 
     * @return a list of client proxies for this stream's foundation baselines
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    ResourceList<CcBaseline> getFoundationBaselineList() throws WvcmException;

    /** The list of components from this stream's complete list of foundation baselines */    
    PropertyName<ResourceList<CcComponent>> COMPONENT_LIST =
        new PropertyName<ResourceList<CcComponent>> (PROPERTY_NAMESPACE,
                                                     "component-list");

    /**
     * Get the value of this stream's {@link #COMPONENT_LIST}
     * property.
     * 
     * @return a list of client proxies for this stream's components that are baselined by this 
     *         stream's foundation baselines
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    ResourceList<CcComponent> getComponentList() throws WvcmException;
    
    /** The default target (stream) for this UCM stream's deliver operations. */
    PropertyName<CcStream> DEFAULT_DELIVER_TARGET =
        new PropertyName<CcStream>(PROPERTY_NAMESPACE,
                                   "default-deliver-target");

    /**
     * Get the value of this stream's {@link #DEFAULT_DELIVER_TARGET} property.
     * 
     * @return a client proxy for this stream's default deliver target,
     *         or <code>null</code> if there is no default deliver target
     *         defined for this stream
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    CcStream getDefaultDeliverTarget() throws WvcmException;

    /**
     * Set the value of this stream's {@link #DEFAULT_DELIVER_TARGET} property.
     * The default deliver target can only be changed on integration streams.
     * The default deliver target for all non-integration streams is their parent 
     * stream.
     * @param target
     *            this stream's new default deliver target or <code>null</code> to clear 
     *            the deliver target
     */
    void setDefaultDeliverTarget(CcStream target);
    
    /** 
     * A list of streams that have posted deliveries in progress to this stream.
     * A delivery is posted if the source stream is mastered at a different 
     * replica than the target stream at the time of the delivery. This property
     * is not recursive; it only includes streams that are immediate children of
     * this stream.
     */
    PropertyName<ResourceList<CcStream>> POSTED_DELIVERY_LIST =
        new PropertyName<ResourceList<CcStream>>(PROPERTY_NAMESPACE, "ccstream-posted-delivery-list");
    
    /**
     * Get the value of this proxy's {@link #POSTED_DELIVERY_LIST} property.
     * @return A list containing this stream's posted deliveries.
     * @throws WvcmException 
     *             if this proxy doesn't define a value for this property.
     */
    ResourceList<CcStream> getPostedDeliveryList() throws WvcmException;

    /** Is this UCM stream in the middle of a delivery?
     */
    PropertyName<Boolean> HAS_DELIVERY_IN_PROGRESS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "has-delivery-in-progress");
    
    /**
     * Get the value of this proxy's {@link #HAS_DELIVERY_IN_PROGRESS} property.
     * @return true if this stream is delivering, else false.
     * @throws WvcmException 
     *             if this proxy doesn't define a value for this property.
     */
    boolean getHasDeliveryInProgress() throws WvcmException;
    
    /** Is this locally mastered UCM stream in the middle of a posted 
     * delivery to a remotely mastered target?
     */
    PropertyName<Boolean> HAS_POSTED_DELIVERY_IN_PROGRESS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "has-posted-delivery-in-progress");
    
    /**
     * Get the value of this proxy's {@link #HAS_POSTED_DELIVERY_IN_PROGRESS} property.
     * @return true if this stream is delivering to a remotely mastered target, else false.
     * @throws WvcmException 
     *             if this proxy doesn't define a value for this property.
     */
    boolean getHasPostedDeliveryInProgress() throws WvcmException;
    
    /**
     * Get the value of this proxy's {@link #DELIVER_OPERATION} proxy.
     * @return deliver operation if there is one in progress, else null
     * @throws WvcmException
     */
    CcDeliverOperation getDeliverOperation() throws WvcmException;
    
    PropertyName<CcDeliverOperation> DELIVER_OPERATION = 
        new PropertyName<CcDeliverOperation>(PROPERTY_NAMESPACE, "deliver-operation");
    
    
    /** Is this UCM stream in the middle of a Rebase?
     */
    PropertyName<Boolean> HAS_REBASE_IN_PROGRESS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "has-rebase-in-progress");
    
    /**
     * Get the value of this proxy's {@link #HAS_REBASE_IN_PROGRESS} property.
     * @return true if this stream is rebasing, else false.
     * @throws WvcmException 
     *             if this proxy doesn't define a value for this property.
     */
    boolean getHasRebaseInProgress() throws WvcmException;    

    /**
     * @deprecated TODO Placeholder for probable future property to track
     * the status of a delivery to the integration stream
     */
    PropertyName INTEGRATION_STATUS = new PropertyName(PROPERTY_NAMESPACE,
                                                       "integration-status");
}